"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const core_1 = require("@yarnpkg/core");
const core_2 = require("@yarnpkg/core");
const fslib_1 = require("@yarnpkg/fslib");
const querystring_1 = __importDefault(require("querystring"));
const tmp_1 = require("tmp");
const constants_1 = require("./constants");
class ExecFetcher {
    supports(locator, opts) {
        if (!locator.reference.startsWith(constants_1.PROTOCOL))
            return false;
        return true;
    }
    getLocalPath(locator, opts) {
        const { parentLocator, execPath } = this.parseLocator(locator);
        if (fslib_1.ppath.isAbsolute(execPath))
            return execPath;
        const parentLocalPath = opts.fetcher.getLocalPath(parentLocator, opts);
        if (parentLocalPath !== null) {
            return fslib_1.ppath.resolve(parentLocalPath, execPath);
        }
        else {
            return null;
        }
    }
    async fetch(locator, opts) {
        const expectedChecksum = opts.checksums.get(locator.locatorHash) || null;
        const [packageFs, releaseFs, checksum] = await opts.cache.fetchPackageFromCache(locator, expectedChecksum, async () => {
            opts.report.reportInfoOnce(core_1.MessageName.FETCH_NOT_CACHED, `${core_2.structUtils.prettyLocator(opts.project.configuration, locator)} can't be found in the cache and will be fetched from the disk`);
            return await this.fetchFromDisk(locator, opts);
        });
        return {
            packageFs,
            releaseFs,
            prefixPath: core_2.structUtils.getIdentVendorPath(locator),
            localPath: this.getLocalPath(locator, opts),
            checksum,
        };
    }
    async fetchFromDisk(locator, opts) {
        const { parentLocator, execPath } = this.parseLocator(locator);
        // If the file target is an absolute path we can directly access it via its
        // location on the disk. Otherwise we must go through the package fs.
        const parentFetch = fslib_1.ppath.isAbsolute(execPath)
            ? { packageFs: new fslib_1.NodeFS(), prefixPath: fslib_1.PortablePath.root, localPath: fslib_1.PortablePath.root }
            : await opts.fetcher.fetch(parentLocator, opts);
        // If the package fs publicized its "original location" (for example like
        // in the case of "file:" packages), we use it to derive the real location.
        const effectiveParentFetch = parentFetch.localPath
            ? { packageFs: new fslib_1.NodeFS(), prefixPath: parentFetch.localPath }
            : parentFetch;
        // Discard the parent fs unless we really need it to access the files
        if (parentFetch !== effectiveParentFetch && parentFetch.releaseFs)
            parentFetch.releaseFs();
        const generatorFs = effectiveParentFetch.packageFs;
        const generatorPath = fslib_1.ppath.resolve(fslib_1.ppath.resolve(generatorFs.getRealPath(), effectiveParentFetch.prefixPath), execPath);
        // Execute the specified script in the temporary directory
        const cwd = await this.generatePackage(locator, generatorPath, opts);
        // Make sure the script generated the package
        if (!fslib_1.xfs.existsSync(fslib_1.ppath.join(cwd, fslib_1.toFilename(`build`))))
            throw new Error(`The script should have generated a build directory`);
        return await core_2.tgzUtils.makeArchiveFromDirectory(fslib_1.ppath.join(cwd, fslib_1.toFilename(`build`)), {
            prefixPath: core_2.structUtils.getIdentVendorPath(locator),
        });
    }
    async generatePackage(locator, generatorPath, opts) {
        const cwd = fslib_1.npath.toPortablePath(tmp_1.dirSync().name);
        const env = await core_2.scriptUtils.makeScriptEnv({ project: opts.project });
        const logFile = fslib_1.npath.toPortablePath(tmp_1.tmpNameSync({
            prefix: `buildfile-`,
            postfix: `.log`,
        }));
        const stdin = null;
        const stdout = fslib_1.xfs.createWriteStream(logFile);
        const stderr = stdout;
        stdout.write(`# This file contains the result of Yarn generating a package (${core_2.structUtils.stringifyLocator(locator)})\n`);
        stdout.write(`\n`);
        const { code } = await core_2.execUtils.pipevp(process.execPath, [fslib_1.npath.fromPortablePath(generatorPath), core_2.structUtils.stringifyIdent(locator)], { cwd, env, stdin, stdout, stderr });
        if (code !== 0)
            throw new Error(`Package generation failed (exit code ${code}, logs can be found here: ${logFile})`);
        return cwd;
    }
    parseLocator(locator) {
        const qsIndex = locator.reference.indexOf(`?`);
        if (qsIndex === -1)
            throw new Error(`Invalid file-type locator`);
        const execPath = fslib_1.ppath.normalize(locator.reference.slice(constants_1.PROTOCOL.length, qsIndex));
        const queryString = querystring_1.default.parse(locator.reference.slice(qsIndex + 1));
        if (typeof queryString.locator !== `string`)
            throw new Error(`Invalid file-type locator`);
        const parentLocator = core_2.structUtils.parseLocator(queryString.locator, true);
        return { parentLocator, execPath };
    }
}
exports.ExecFetcher = ExecFetcher;
